#ifndef SLAM_DECISION_H_
#define SLAM_DECISION_H_

#include <iostream>
#include <vector>

namespace slam
{

    enum RoadType : uint8_t
    {
        NOT_AVAILABLE = 0,
        RAMP_SPLIT = 1,               // "匝道内分岔路"
        HIGHWAY_CONNECTOR = 2,        // "高速连接路"
        HIGHWAY_GROUND_CONNECTOR = 3, // "高速地面连接路"
        MAIN_ROAD_FORK = 4,           // "主路分岔路"
        RAMP_MERGING_MAIN_ROAD = 5,   // "匝道汇合主路"
        MAIN_ROAD_MERGING_ROAD = 6,   // "主路汇合路"
        VIRTUAL_MAIN_ROAD = 7,        // "虚拟主路"
        RMD_SPLIT_ROAD = 8,           // "提醒用主路匝道分岔路，主路径为主路"
        RMD_MERGE_ROAD = 9            // "提醒用匝道汇合主路，主路径为主路"
    };

    struct AdjacentExtendLaneStruct
    {
        uint8_t adjacent_extendlane_id;      // Lane ID
        float adjacent_extendlane_offset;    // Starting position offset from the vehicle
        float adjacent_extendlane_endoffset; // Ending position offset from the vehicle
    };

    struct SMAStruct
    {
        uint8_t lane_id;    // Lane ID
        uint8_t type;       // odd type
        uint8_t sub_type;   // odd subtype
        float start_offset; // starting position offset of odd
        float end_offset;   // ending position offset of odd
    };
    struct WPExtendLaneStruct
    {
        uint8_t lane_id;                                         // lane id
        //uint8_t padding[3];                                      // padding
        float lane_latest_lc_point_offset_to_wp;                 // Latest lane change point
        std::vector<AdjacentExtendLaneStruct> left_extend_lane;  // List of left adjacent lanes
        std::vector<AdjacentExtendLaneStruct> right_extend_lane; // List of right adjacent lanes
        uint8_t lane_priority;                                   // lane priority
        uint8_t is_emergency_lane;                               // certain lane type
        float current_extendlane_offset;                         // starting position offset of current lane
        float current_extendlane_endoffset;                      // ending position offset of current lane
        std::vector<SMAStruct> sma_info;                         // odd info of current lane
    };

    struct TransitLaneStruct
    {
        uint8_t transit_type;              // Lane Transition Type
        uint8_t transit_lane_id;           // Lane ID
        //uint8_t padding[2];                // padding
        float transit_lane_offset;         // Distance to lane transition point
        uint8_t transit_connected_lane_id; // Connected lane id
    };
    struct RemindWaypointStruct
    {
        uint8_t reminder_waypoint_target_road_type;                        // Reminder waypoint type
        uint32_t reminder_waypoint_id;                                     // Reminder waypoint ID
        std::vector<uint8_t> reminder_waypoint_lane_id_list;               // Target lanes of remind waypoint
        float reminder_waypoint_offset;                                    // Reminder waypoint distance in meters
        std::vector<AdjacentExtendLaneStruct> reminder_wp_extendlane_list; // Related extend lane list
        uint8_t reminder_waypoint_speed_limit;                             // Speed limit at reminder waypoint
    };
    struct ControlWaypointStruct
    {
        RoadType control_waypoint_target_road_type;                 // Control waypoint type
        uint32_t control_waypoint_id;                               // Control waypoint ID
        std::vector<uint8_t> control_waypoint_lane_id_list;         // Target lane list of control waypoint
        std::vector<uint8_t> control_waypoint_raw_lane_id_list;     // Original target lanes list
        float control_waypoint_offset;                              // Distance to control waypoint in meters
        std::vector<WPExtendLaneStruct> control_wp_extendlane_list; // Related extend lane list
        uint8_t deceleration_lane_type;                             // Deceleration lane type
        uint8_t control_waypoint_speed_limit;                       // Speed limit at control waypoint
    };
    
    struct MsgHeader
    {
        uint64_t time_stamp;    // Timestamp
        uint16_t cycle_counter; // Cycle Counter
        //uint8_t padding[2];     // padding
    };

    struct ControlWaypoints
    {
        MsgHeader msg_header;                              // Message Header
        std::vector<ControlWaypointStruct> control_waypoints_list;   // List of Control Way Points
        std::vector<RemindWaypointStruct> reminder_waypoint;         // Reminder Way Point
        std::vector<TransitLaneStruct> transit_lanes_list; // Transit Lane List
        float remainDistanceMajorPath;                     // Remaining Distance of Major Path
    };

}
#endif // SLAM_DECISION_H_
